<?php

use Slim\{
    Psr7\Request,
    Psr7\Response,
    Psr7\Factory\ResponseFactory,
    Csrf\Guard,
    Views\PhpRenderer,
    Middleware\ErrorMiddleware
};
use Dotenv\Dotenv;
use DI\ContainerBuilder;
use Zeuxisoo\Whoops\Slim\WhoopsMiddleware;
use App\response\errorResponse;
use App\middleware\{
    routelogMiddleware,
    ipaccessMiddleware
};
use RKA\Middleware\IpAddress as ipaddressMiddleware;

$activephpversion=phpversion();
if (version_compare($activephpversion, '7.3.0', '<')) {
    echo "You are running PHP version \"<strong>$activephpversion</strong>\", but needs at least PHP \"<strong>7.3.0</strong>\" to run, upgrade your PHP installation.";
    exit;
}
if (!function_exists('json_encode')) {
    echo 'json_encode() must be available install and enable JSON extension.';
    exit;
}

require_once (__DIR__ . '/vendor/autoload.php');

$cmfrun='admin';
$configenvpath=base_path('/angiv/config/');
if (is_file($configenvpath . 'config.env') && is_readable($configenvpath . 'config.env')) {
    Dotenv::createImmutable($configenvpath, 'config.env')->load();
    date_default_timezone_set(env('APP_TIMEZONE', 'America/Los_Angeles'));
} else {
    $cmfrun='installer';
}
$_ENV['APP_VERSION']='0.4';
$_ENV['APP_CMFRUN']=$cmfrun;
$_ENV['APP_ADMIN_URL']=str_replace('//', '/', '/' . env('APP_ADMIN_URL', 'xadmin'));
$_ENV['APP_AUTH_URL']=str_replace('//', '/', '/' . env('APP_AUTH_URL', 'welcome'));
if (empty($_ENV['APP_AUTH_USE'])) {$_ENV['APP_AUTH_USE']='true';}

$sessionpath=base_path('/angiv/tmp/session/');
if (is_dir($sessionpath) && is_readable($sessionpath)) {
    ini_set('session.save_handler', 'files');
    ini_set('session.save_path', $sessionpath);
}
/*
ini_set('display_errors', 0);
ini_set('display_startup_errors', 0);
error_reporting(E_ALL);
*/

$builder=new ContainerBuilder();
if (envbool('APP_CACHE_USE') === true) {
    $builder->enableCompilation(base_path('/angiv/tmp/cache/container/'));
    $builder->writeProxiesToFile(true, base_path('/angiv/tmp/cache/proxy/'));
}
$builder->addDefinitions(base_path('/app/config/container.php'));
$container=$builder->build();

$app=$container->get('app');
if (envbool('APP_CACHE_USE') === true) {$app->getRouteCollector()->setCacheFile(base_path('/angiv/tmp/cache/route/route.php'));}
$currenturl=$container->get('uri')['path'];

$logger=null;
if (empty($_ENV['APP_LOG_USE']) || envbool('APP_LOG_USE') === true) {$logger=$container->get('logger');}

$app->add($container->get('session'));
$app->addBodyParsingMiddleware();
$app->add(new ipaddressMiddleware(true, [], 'ipaddress', ['X-REAL-IP', 'FORWARDED', 'X-FORWARDED', 'X_FORWARDED', 'FORWARDED_FOR', 'X-FORWARDED-FOR', 'X_FORWARDED_FOR', 'X_FORWARDED_FOR_IP', 'FORWARDED_FOR_IP', 'X-CLUSTER-CLIENT-IP', 'CLIENT-IP', 'CLIENT_IP', 'VIA', 'COMING_FROM', 'X_COMING_FROM', 'CF-CONNECTING-IP', 'TRUE-CLIENT-IP', 'X_IMFORWARDS', 'XROXY_CONNECTION', 'PROXY_CONNECTION', 'PC_REMOTE_ADDR']));
if ($cmfrun != 'installer' && $currenturl != '/403') {
    $ipwhitelist=[]; if (!empty($_ENV['APP_PERM_IP_WHITE'])) {$ipwhitelist=array_fill_keys(array_map('trim', explode(',', $_ENV['APP_PERM_IP_WHITE'])), 1);}
    $ipblacklist=[]; if (!empty($_ENV['APP_PERM_IP_BLACK'])) {$ipblacklist=array_fill_keys(array_map('trim', explode(',', $_ENV['APP_PERM_IP_BLACK'])), 0);}
    $app->add(new ipaccessMiddleware(array_replace($ipwhitelist, $ipblacklist)));
}
if (envbool('APP_LOG_ROUTES') === true && !empty($logger)) {$app->add(new routelogMiddleware($app->getRouteCollector(), $logger));}
if (!empty($usermiddleware=@include base_path('/angiv/middleware/userMiddleware.php'))) {$usermiddleware($app);}

$app->get('/401', function (Request $request, Response $response) {
    $msg=__('Error authentication');
    $renderer=new PhpRenderer(base_path('/angiv/theme/page/'), ['title' => $msg]);
    return $renderer->render($response, '401.php', compact('msg'));
});

$app->get('/403', function (Request $request, Response $response) {
    $renderer=new PhpRenderer(base_path('/angiv/theme/page/'), ['title' => __('Access denied')]);
    $msg=__('Access denied');
    return $renderer->render($response, '403.php', compact('msg'));
});

$app->get('/404', function (Request $request, Response $response) {
    $renderer=new PhpRenderer(base_path('/angiv/theme/page/'), ['title' => __('Page not found')]);
    $msg=__('You have found a page that does not exist.');
    $question=__('Where do we go next?');
    $back=__('Go back to main page');
    return $renderer->render($response, '404.php', compact('msg', 'question', 'back'));
});

if (!in_array($currenturl, ['/401', '/403', '/404'])) {
    if (str_starts_with($currenturl, '/auth') !== false) {
        if ($cmfrun == 'admin' && envbool('APP_AUTH_USE') === true) {
            require_once base_path('/app/admin/routeauth.php');
        }
    } else {
        require_once base_path("/app/{$cmfrun}/route.php");
    }
}

if (envbool('APP_DEBUG_USE') === true) {
    if (envbool('APP_DEBUG_WHOOPS') === true) {
        $app->add(new WhoopsMiddleware(['title' => __('Whoops, looks like something went wrong')], [
            function ($exception, $inspector, $run) use ($logger) {
                if (!empty($logger)) {
                    $errdata=['code' => $exception->getCode(), 'class' => get_class($exception), 'message' => explode('|csrf', $exception->getMessage(), 2)[0], 'file' => $exception->getFile(), 'line' => $exception->getLine()];
                    $logger->error($errdata['code'] . (!empty($errdata['message']) ? (', ' . $errdata['message']) : ''), $errdata);
                }
            }
        ]));
    } else {
        $errorMiddleware=$app->addErrorMiddleware(true, true, true, $logger);
        $errorMiddleware->getDefaultErrorHandler()->registerErrorRenderer('text/html', errorResponse::class);
    }
} else {
    $app->addErrorMiddleware(false, false, false)->setDefaultErrorHandler(
        function (Request $request, Throwable $exception, bool $displayErrorDetails, bool $logErrors, bool $logErrorDetails) use ($logger) {
            if (!empty($logger)) {
                $errdata=['code' => $exception->getCode(), 'class' => get_class($exception), 'message' => explode('|csrf', $exception->getMessage(), 2)[0], 'file' => $exception->getFile(), 'line' => $exception->getLine()];
                $logger->error($errdata['code'] . (!empty($errdata['message']) ? (', ' . $errdata['message']) : ''), $errdata);
            }
            $response=new Response(); //$response=$app->getResponseFactory()->createResponse();
            if (strtoupper($request->getMethod()) === 'POST') {
                $phpcsrf=implode('|', (new Guard(new ResponseFactory()))->generateToken());
                $response->getBody()->write((string)json_encode(['message' => __('Ajax-request error, try again later'), 'csrf' => $phpcsrf], JSON_UNESCAPED_SLASHES | JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE/* | JSON_NUMERIC_CHECK*/ | JSON_PRESERVE_ZERO_FRACTION));
                return $response->withHeader('Content-Type', 'application/json')->withStatus(400);
            } else {
                return $response->withHeader('Location', '/404')->withStatus(301);
            }
        }
    );
}